// This is the Arduino Sketch for the ESP32 Multicolor Word Clock by Professor Ziebart
// Ich übernehme keinerlei Haftung für irgendwas, Zerstörung von Chips, Epileptische Anfälle, gestorbene Elfen und dergleichen.
// Diese Software ist lizenziert unter Creative Commons unter Nennung des Inhabers.
// Ganz großen Dank an Rui Santos, der mich hierzu z.T. inspiriert hat und auf dessen Arbeit dieser Sketch aufbaut. 
// I do not take any responsabilty for anything, destroying of Microcontrollers, epileptic Attacs, died Elves or whatsoever
// This Software is shared under Creative Commmons License Attribution Sharealike
// Feel Free to embed new "Color Modes" 
// Many thanks to Rui Santos from www.randomnerdtutorials on which this Work is rooted on.


  */
// Includes First //////////////////////////////////////////////////////////////////
#include <WiFi.h>               // For the Wifi-Connection
#include <AsyncTCP.h>           // For the Asynchronous TCP Server
#include <ESPAsyncWebServer.h>  // the asynchronous Webserver of Rui Santos
#include <time.h>               // for Time Calculations
#include <Adafruit_NeoPixel.h>  // for the Pixels...


// Constants Second ////////////////////////////////////////////////////////////////
#define WifiPWD               "YourPWDORHASH" // the Hashed Wifipassword
#define WifiSSID              "YOURSSID" // the SSID of the targetet WIfI Network
#define PARAM_INPUT_1         "para" // the Mode Variable for the Webserver
#define PARAM_INPUT_2         "val"   // the First Parameter for the Webserver
#define ntpServer             "pool.ntp.org" // We get a Server from ntp.org
#define gmtOffset_sec         3600 // Timezone Offset +1h
#define daylightOffset_sec    3600 // Whatever this is. Maybe Summertime?
#define LEDSPIN               15  // The Pin where the 470k Resistor leads to the First DIN of the LEDs
#define MAXLEDS               100  // the Number of Neopixels attached. Cannot be defined while runtime.
const char DELIMITER[2]  = ","; // the Delimiter for the RGB-Colorvalue
// the Webpage Header is some Kind of Long Constant ////////////////////////////////
const char index_html[] PROGMEM = R"rawliteral(
<!DOCTYPE HTML><html>
<head>
  <title>Multicolor Word Clock by Prof. Ziebart</title>
  <meta name="viewport" content="width=device-width, initial-scale=1">
  <link rel="icon" href="data:,">
  <style>
    html {font-family: Arial; display: inline-block; text-align: center;}
    h2 {font-size: 3.0rem;}
    p {font-size: 2.0rem;}
    body {max-width: 600px; margin:0px auto; padding-bottom: 20px;}
    .switch {position: relative; display: inline-block; width: 120px; height: 68px} 
    .switch input {display: none}
    .slider {position: absolute; top: 0; left: 0; right: 0; bottom: 0; background-color: #ccc; border-radius: 6px}
    .slider:before {position: absolute; content: ""; height: 52px; width: 52px; left: 8px; bottom: 8px; background-color: #fff; -webkit-transition: .4s; transition: .4s; border-radius: 3px}
    input:checked+.slider {background-color: #b30000}
    input:checked+.slider:before {-webkit-transform: translateX(52px); -ms-transform: translateX(52px); transform: translateX(52px)}
    .slider2 { -webkit-appearance: none; margin: 14px; width: 360px; height: 25px; background: #ccc;
      outline: none; -webkit-transition: .2s; transition: opacity .2s;}
    .slider2::-webkit-slider-thumb {-webkit-appearance: none; appearance: none; width: 35px; height: 35px; background: #003249; cursor: pointer;}
    .slider2::-moz-range-thumb { width: 35px; height: 35px; background: #b30000; cursor: pointer; } 
    .box {width: 120px; height: 30px; border: 2px solid #999; font-size: 18px; color: #b30000; background-color: #ccc; border-radius: 5px;}
   </style>
</head>
<body>
  <h2>Multicolor Word Clock <br> Control </h2>
  %BUTTONPLACEHOLDER%
<script>
function toggleCheckbox(element) {
  var xhr = new XMLHttpRequest();
  if(element.checked){ xhr.open("GET", "/update?para="+element.id+"&val=1", true); }
  else { xhr.open("GET", "/update?para="+element.id+"&val=0", true); }
  xhr.send();
}
function updateSlider(element) {
  var SliderValue = element.value;
  if (element.id == "MinLED") {
  document.getElementById("MinLEDText").innerHTML = SliderValue;
  }
  if (element.id == "MaxLED") {
  document.getElementById("MaxLEDText").innerHTML = SliderValue;
  }
  if (element.id == "LEDBrightness") {
  document.getElementById("LEDBrightnessText").innerHTML = SliderValue;
  }
  var xhr = new XMLHttpRequest();
  xhr.open("GET", "/update?para="+element.id+"&val="+SliderValue, true);
  xhr.send();
}
function updateColorPicker(element) {
  var Color1Value = element.value;
  var Color1Value = Color1Value.replace(/#/g, '');
  if (Color1Value.length === 3) {
        Color1Value = Color1Value.split('').map(function (Color1Value) {
            return Color1Value + Color1Value;
        }).join('');
    }
    var Color1Value = /^([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})[\da-z]{0,0}$/i.exec(Color1Value);
    if (Color1Value) {
        var red = parseInt(Color1Value[1], 16);
        var green = parseInt(Color1Value[2], 16);
        var blue = parseInt(Color1Value[3], 16);

        Color1RGBValue = [red, green, blue];
    } 
  document.getElementById("Color1Text").innerHTML = Color1Value + " oder " + Color1RGBValue;
  var xhr = new XMLHttpRequest();
  xhr.open("GET", "/update?para="+element.id+"&val="+Color1RGBValue, true);
  xhr.send();
}

function updateCombo(element) {
  var ComboValue = element.value;
  if (element.id == "ComboModus") {
  document.getElementById("ModusText").innerHTML = ComboValue;
  }
  var xhr = new XMLHttpRequest();
  xhr.open("GET", "/update?para="+element.id+"&val="+ComboValue, true);
  xhr.send();
}
</script>
</body>
</html>
)rawliteral";

// Variables Third /////////////////////////////////////////////////////////////////
String      Slider1Value ="0";               // the Value of the ParameterSlider
boolean     RUonline= false;                 // the Online Status. False if offline, True if online
int         i=50;                             // the Superstar of Counting Variables
char StartuptimeHour[3];                     // the Hour we got NTP Time Last
struct tm timeinfo;                          // Some NTP ESP32 Stuff. Time pointer.
char timeHour[3] = "NN";                     // the current Hour
char timeHourTwelve[3];                      // the current Hour in 12 Hour format
char timeMinute[3];                          // the current Minute
char timeSecond[3];                          // the current Second
uint8_t  LEDBrightness = 30;                     // the LED Brighntess for starters
uint16_t  LEDMax = 100;                            // the Number of Next LEDs Complete Max is a constant.
uint16_t  LEDMin = 0;                             // the Number of the First LED to test
uint8_t  LEDMode = 0;                        // the Displaymode. 0 is the Testmode.
String LEDColorValue = "308090";             // the Color Value
uint8_t LEDRed = 0;                          // A possible red Color Value
uint8_t LEDGreen = 0;                        // A possible green Color Value
uint8_t LEDBlue = 0;                         // A possible blue Color Value
String WholeTimeinWords ="";                 // the Whole Time in Words more or less just for debugging
String ItIs = "Es ist";                      // the String containing the "It is"
uint8_t ItIsInt[5] = {1,2,4,5,6};            // the Integer Array contaning the "It is" LEDs.
String Minuteword = "nix";                   // the String containing the Minuteword e.g. "five"
uint8_t MinuteInt[7] = {0,0,0,0,0,0,0};      // the Integer Array contaning the "five" LEDs.
String ToPastWord = "vor";                   // the String contaning the To or Past e.g. "to"
uint8_t ToPastInt[4] = {0,0,0,0};            // the Integer Array contaning the "to" LEDs.
String HalfWord = "halb";                    // the String containing the Halfword, e.g. "half past"
uint8_t HalfInt[4] = {0,0,0,0};              // the Integer Array contaning the "It is" LEDs.
String HourWord = "nix";                     // the String containing the Hourword e.g. "Twelve".
uint8_t HourInt[6] = {0,0,0,0,0,0};          // the Integer Array contaning the Hour LEDs.6 with max 12 and 14 with max 24 Hours
bool   TimeWordChanged = "true";             // Boolean showing if the Words have changed.
String TimeOutput="";                        // The Time in Words
char str[12];                                // A Char Array necessary for converting the RGB String to the Int Values
bool Triggeronce = true;                     // A Boolean which means that an Action has been triggered. So the LED Mode fires only once.
uint8_t Colorhelper = 0;                      // A cycling Variable for the COlorwheel.

// Was will ich eigentlich weiter haben:
// Ein Array mit den LEDs, die ein Wort ausmachen
// EIn Array mit allen WortLEDs
// Ein Array mit allen "NICHTWORT" LEDs

// Objects Fourth //////////////////////////////////////////////////////////////////
AsyncWebServer server(80);                  // Create an Asynchronous Webserver
Adafruit_NeoPixel strip = Adafruit_NeoPixel(MAXLEDS, LEDSPIN, NEO_GRB + NEO_KHZ800);

// Finally Setup //////////////////////////////////////////////////////////////////
void setup() {
   // // // Setup the Serial Connection for Debugging
  Serial.begin(115200);
  while(!Serial){
    Serial.print('.');
  }
  Serial.println(" Hello! I am alive!");
  // // // Setup the Wifi Mode and Connection
  WiFi.begin(WifiSSID, WifiPWD);
    Serial.println("Connecting..");
    if (WiFi.waitForConnectResult() != WL_CONNECTED) {
        Serial.print(".");
        while(1) {
            delay(1000);
        }
    }
    else {
     // We are online!
    Serial.println("Done. Connected.");
    RUonline = true;     
    // Show the local IP ....
    Serial.print("My local IP Adress is :");
    Serial.println(WiFi.localIP());
    delay(1000);
    //We are ready to rumble.. Connected to Wifi /////////////////////
    // // // We define the Webserver Request.
    server.on("/", HTTP_GET, [](AsyncWebServerRequest *request){
        request->send_P(200, "text/html", index_html, processor);
        });
    // Send a GET request to <ESP_IP>/update?output=<inputMessage1>&state=<inputMessage2>
    server.on("/update", HTTP_GET, [] (AsyncWebServerRequest *request) {
      String inputMessage1;
      String inputMessage2;
      // GET input1 value on <ESP_IP>/update?para=<inputMessage1>&val=<inputMessage2>
      if (request->hasParam(PARAM_INPUT_1) && request->hasParam(PARAM_INPUT_2)) {
         inputMessage1 = request->getParam(PARAM_INPUT_1)->value();
         inputMessage2 = request->getParam(PARAM_INPUT_2)->value();
         //Executing Actions because the Webinterface has changed.
         WebAction(inputMessage1, inputMessage2);
         }
       else {
         inputMessage1 = "No message sent";
         inputMessage2 = "No message sent";
         }
       Serial.print("Parameter: ");
       Serial.print(inputMessage1);
       Serial.print(" - Set to Value: ");
       Serial.println(inputMessage2);
       request->send(200, "text/plain", "OK");
       });
       // Start server
       server.begin();
       //init and get the time
       configTime(gmtOffset_sec, daylightOffset_sec, ntpServer);
       }
       GetNTPTime();
       TimeWordChanged = true; // We pretend to have a new Timeword to trigger Timeshowing behaviour
       // Initialise the LED Strip
       strip.setBrightness(LEDBrightness);
       strip.show();
       
}

// the Main Loop. //////////////////////////////////////////////////////////////////////////////////////////
void loop() {
if (Colorhelper > 256) {Colorhelper = 0;}
if ( i >= 100) {
   GetTime(); // Gets the Time from internal clock.
   i = 0; 
   ConvertTime(); // If the Time changes so that new Words light up TimeWordChanged gets set to true there.
   }
i += 1;
if (TimeWordChanged == true) {
  // the Time has changed.
  TimeOutput = ItIs + " " + Minuteword + " " + ToPastWord + " " + HalfWord + " " + HourWord;
  Serial.println("   ---   ");
  Serial.println(&timeinfo, "%A, %B %d %Y %H:%M:%S");
  Serial.println(TimeOutput);
  MapWordstoPixel();
  }
// Now we trigger the Wanted Mode.
if ((LEDMode == 0) && (LEDMin < LEDMax)){
   LEDTest(LEDMin, LEDMax, LEDBrightness, LEDColorValue);
 }
if ((LEDMode == 1)&& (TimeWordChanged == true)){
   MonoColorClock(LEDColorValue);
   TimeWordChanged = false;
 }
if ((LEDMode == 2) && Triggeronce){
   MultiColorClock(LEDMin, Colorhelper );   
   TimeWordChanged= false;
 
 }
if ((LEDMode == 3)){
   RainbowColorClock(LEDMin, Colorhelper, ((LEDMax +10)/10) );
   TimeWordChanged = false;
 }
if ((LEDMode == 4)){
   //theaterChaseRainbow(10, LEDMin, LEDMax);
  //  rainbowCycle(10, LEDMin, LEDMax);
  //Triggeronce = false;
  InvertedColorClock(LEDMin, Colorhelper);
 }
if ((LEDMode == 5) && Triggeronce){
   SingleLED(LEDMin, LEDMax, LEDColorValue);
   Triggeronce = false;
 }
 
Colorhelper++; // Set the next Cyclecolor!
TimeWordChanged = false;
}

// the Processor processes the Webpage....
String processor(const String& var){
  //Serial.println(var);
  if(var == "BUTTONPLACEHOLDER"){
    String buttons = "";
    // Oben schreibe ich die aktuelle Zeit rein.
    if (timeHour !="NN"){
      // Wenn die timeHour none ist, haben wir noch keine Zeit, wir haben also eine aktuelle Zeit. 
      buttons += "<h4> Es ist jetzt ";
      buttons.concat(timeHour);
      buttons +=  " : "; 
      buttons.concat(timeMinute);
      buttons +=  " : ";
      buttons.concat(timeSecond);
      buttons +=  ". <br>";
      buttons += TimeOutput;
    }
    buttons += "<h4>An/Aus </h4><label class=\"switch\"><input type=\"checkbox\" onchange=\"toggleCheckbox(this)\" id=\"OnOff\" " + outputState(2) + "><span class=\"slider\"></span></label>";
    buttons += "<h4>Start LED :  <span id=\"MinLEDText\">%MINLEDVALUE%</span> <\h4> <p><input type=\"range\" onchange=\"updateSlider(this)\" id=\"MinLED\" min=\"0\" max=\"100\" value=\"%MINLEDVALUE%\" step=\"1\" class=\"slider2\"></p>";
    buttons += "<h4>Ende LED :  <span id=\"MaxLEDText\">%MAXLEDVALUE%</span> <\h4> <p><input type=\"range\" onchange=\"updateSlider(this)\" id=\"MaxLED\" min=\"1\" max=\"100\" value=\"%MAXLEDVALUE%\" step=\"1\" class=\"slider2\"></p>";
    buttons += "<h4>Brightness : <span id=\"LEDBrightnessText\">%LEDBRIGHTNESSVALUE%</span> <\h4> <p><input type=\"range\" onchange=\"updateSlider(this)\" id=\"LEDBrightness\" min=\"1\" max=\"255\" value=\"%LEDBRIGHTNESSVALUE%\" step=\"1\" class=\"slider2\"></p>";
    buttons += "<h4>Colorpicker: <span id=\"Color1Text\">%COLOR1VALUE%</span> <\h4> <p><input type=\"color\" onchange=\"updateColorPicker(this)\" id=\"ColorPicker1\" value=\"%COLOR1VALUE%\"></p>";
    buttons += "<h4>Modus: <span id=\"ModusText\">%COMBOMODUSVALUE%</span> <\h4> <p><select onchange=\"updateCombo(this)\"id=\"ComboModus\" class=\"box\"><option value=0>Testmodus</option><option value=1>Einfarbige Wordclock</option><option value=2>Mehrfarbige Wordclock</option><option value=3>Regenbogen Words</option><option value=4>RegenbogenBackground</option><option value=5>Zs Wordclock</option></select></p>";
   return buttons;
  }
  return String();
}
// outputState is a helper for the processor
String outputState(int output){
  if(digitalRead(output)){
    return "checked";
  }
  else {
    return "";
  }
}
// GetTime gets the Internal Time
void GetTime() {
  getLocalTime(&timeinfo);
  if (timeHour == "NN"){
    // We have not obtained ANY Time yet.
    GetNTPTime();
    Serial.println("Getting new NTP Time. We have none yet.");
    }
  else {
 //   Serial.print("Using current local time :");
    // We have some Time info. So we can source out of the internal Time.  
    if (StartuptimeHour == timeHour-1) {
        // Some time has passed since we obtained Internet Time.
        Serial.println("Getting new NTP Time. The old one is outdated");
        GetNTPTime();
       }
    // We have a valid Time info and take the internal time.

//    Serial.print("Hour: ");
    strftime(timeHour,3, "%H", &timeinfo);
//    Serial.print(timeHour);
//    Serial.print(" / ");
//    Serial.print("Hour (12 hour format): ");
    strftime(timeHourTwelve,3, "%I", &timeinfo);
//    Serial.print(timeHour);
//    Serial.print(" / ");
//    Serial.print("Minute: ");
    strftime(timeMinute,3, "%M", &timeinfo);
//    Serial.print(timeMinute);
//    Serial.print(" / ");
//    Serial.print("Second: ");
    strftime(timeSecond,3, "%S", &timeinfo);
//    Serial.println(timeSecond);
    }
}

// GetNTPTime gets the NTP TIme if we don't have an internal clock set.
void GetNTPTime()
// This Routine is just for getting Internet Time
{
  Serial.println("NTP Time fetching started");
  if(!getLocalTime(&timeinfo)){
    Serial.println("Failed to obtain time");
      // The Following Command gets the NTP Time;
      if (RUonline) {
        configTime(gmtOffset_sec, daylightOffset_sec, ntpServer);
        // We set the Startuptimehour to be able to determine WHEN we last got Time info from NTP
        strftime(StartuptimeHour,3, "%H", &timeinfo);
        strftime(timeHour,3, "%H", &timeinfo);
        }
        else {
          Serial.println("I have no time info and I am not online...");
        }
    return;
    } else
    {
   strftime(timeHour,3, "%H", &timeinfo);
   strftime(timeHourTwelve,3, "%I", &timeinfo);
   strftime(timeMinute,3, "%M", &timeinfo);
   strftime(timeSecond,3, "%S", &timeinfo);
   }
}

// WebAction is triggered when something happens on the Webpage. Then we rewrite the local Variables.
void WebAction(String Para  , String Value) {
Serial.println("I have a webaction :");
Serial.print(Para);
Serial.print(" is ");
Serial.println(Value);
if (Para == "OnOff"){
  //ON OFF STATEMENTS Has Parameters 0 and 1
    if (Value == "0"){ 
      // 1 Turns the Webserver off. To Restart, Restart ESP32. Powersaving.
      Serial.println("Shutdown the asynchronous Webserver.");
      server.end();
      // Shutdown WLAN.
      Serial.println("Now Disconnect from WiFi.");
      WiFi.disconnect(true);
      Serial.println("Now turn off WiFI.");
      WiFi.mode(WIFI_OFF);
      }
    }
if (Para == "MinLED"){
  LEDMin = Value.toInt();

  Serial.print("LEDMin is now:");
  Serial.println(LEDMin);
  }
if (Para == "MaxLED"){
  LEDMax = Value.toInt();
  Serial.println("LEDMax is now:");
  Serial.println(LEDMax);
  }
if (Para == "LEDBrightness"){
  LEDBrightness = Value.toInt();
  Serial.println("Brightness is now:");
  Serial.println(LEDBrightness);
  }
 
if (Para == "ColorPicker1"){
  LEDColorValue = Value;
  Serial.println("ColorValue is now:");
  Serial.println(LEDColorValue);
  Colorval2RGB(LEDColorValue, LEDRed, LEDGreen, LEDBlue);
  }
if (Para == "ComboModus"){
  LEDMode = Value.toInt();
  Serial.print("LEDMode is now ");
  Serial.println(LEDMode);
  }
  // the ComboBox Values:
  // 0 - Testmode
  // 1 - OneColor Wordclock
  // 2 - Multicolor Wordclock
  // 3 - Rainbow Words
  // 4 - Inverted Rainbow: Words Black, Background Rainbow
  // 5 - my Special Mode with Waterdrops
// Set the Trigger to perform an Action!
Triggeronce = true;
// Trigger Wordclockactions
TimeWordChanged = true;
}

// ConvertTime converts the Time to Words.
void ConvertTime() {
// Hier konvertiere ich die Zeit in die Wörter
// Es ist fünf vor Halb Zwölf.
// Minuteword = "nix";                   // the String containing the Minuteword e.g. "five"
// ToPast = "vor";                       // the String contaning the To or Past e.g. "to"
// HalfWord = "halb";                    // the String containing the Halfword, e.g. "half past"
// HourWord
int Minute = 0;
Minute = CharToInt(timeMinute[0], timeMinute[1]);
if (Minute > 0 && Minute < 5){
// Serial.print("es ist 0 bis 5 nach "); 
 if (Minuteword != "kurz" || ToPastWord != "nach" || HalfWord != "") {
    Minuteword = "kurz";
    ToPastWord = "nach";
    HalfWord = "";
    TimeWordChanged = true;
    }
 }
if (Minute >= 5 && Minute < 10){
// Serial.print("es ist 5 bis 10 nach ");
  if (Minuteword != "fünf" || ToPastWord != "nach" || HalfWord != "") {
    Minuteword = "fünf";
    ToPastWord = "nach";
    HalfWord = "";
    TimeWordChanged = true;
    }
 }
if (Minute >= 10 && Minute < 15){
// Serial.println("es ist 10 bis 15 nach"); 
 if (Minuteword != "zehn" || ToPastWord != "nach" || HalfWord != "") {
    Minuteword = "zehn";
    ToPastWord = "nach";
    HalfWord = "";
    TimeWordChanged = true;
    }
 }
if (Minute >= 15 && Minute < 20){
// Serial.println("es ist 15 bis 20 nach");
 if (Minuteword != "viertel" || ToPastWord != "nach" || HalfWord != "") {
    Minuteword = "viertel";
    ToPastWord = "nach";
    HalfWord = "";
    TimeWordChanged = true;
    } 
 } 
if (Minute >= 20 && Minute < 25){
// Serial.println("es ist 20 bis 25 nach"); 
 if (Minuteword != "zehn" || ToPastWord != "vor" || HalfWord != "halb") {
    Minuteword = "zehn";
    ToPastWord = "vor";
    HalfWord = "halb";
    TimeWordChanged = true;
   }
 }
if (Minute >= 25 && Minute < 30){
// Serial.println("es ist 25 bis 29 nach"); 
 if (Minuteword != "fünf" || ToPastWord != "vor" || HalfWord != "halb") {
    Minuteword = "fünf";
    ToPastWord = "vor";
    HalfWord = "halb";
    TimeWordChanged = true;
    }
 }
if (Minute == 30){
// Serial.println("es ist 30 nach");
 if (Minuteword != "" || ToPastWord != "" || HalfWord != "halb") {
    Minuteword = "";
    ToPastWord = "";
    HalfWord = "halb";
    TimeWordChanged = true;
    } 
 }
if (Minute > 30 && Minute < 35){
// Serial.println("es ist 30 bis 35 nach");
 if (Minuteword != "kurz" || ToPastWord != "nach" || HalfWord != "halb") {
    Minuteword = "kurz";
    ToPastWord = "nach";
    HalfWord = "halb";
    TimeWordChanged = true;
    } 
 }
if (Minute >= 35 && Minute < 40){
// Serial.println("es ist 35 bis 40 nach");
 if (Minuteword != "fünf" || ToPastWord != "nach" || HalfWord != "halb") {
    Minuteword = "fünf";
    ToPastWord = "nach";
    HalfWord = "halb";
    TimeWordChanged = true;
    } 
 } 
if (Minute >= 40 && Minute < 45){
// Serial.println("es ist 35 bis 40 nach");
 if (Minuteword != "zehn" || ToPastWord != "nach" || HalfWord != "halb") {
    Minuteword = "zehn";
    ToPastWord = "nach";
    HalfWord = "halb";
    TimeWordChanged = true;
    } 
 } 
if (Minute >= 45 && Minute < 50){
// Serial.println("es ist 45 bis 50 nach"); 
 if (Minuteword != "viertel" || ToPastWord != "vor" || HalfWord != "") {
    Minuteword = "viertel";
    ToPastWord = "vor";
    HalfWord = "";
    TimeWordChanged = true;
    } 
 }
if (Minute >= 50 && Minute < 55){
// Serial.println("es ist 50 bis 55 nach");
  if (Minuteword != "zehn" || ToPastWord != "vor" || HalfWord != "") {
    Minuteword = "zehn";
    ToPastWord = "vor";
    HalfWord = "";
    TimeWordChanged = true;
    }  
 } 
if (Minute >= 55 && Minute <= 59){
// Serial.println("es ist 55 bis 59 nach"); 
  if (Minuteword != "fünf" || ToPastWord != "vor" || HalfWord != "") {
    Minuteword = "fünf";
    ToPastWord = "vor";
    HalfWord = "";
    TimeWordChanged = true;
    } 
 } 
if (Minute == 0){
// Serial.println("es ist genau"); 
  if (Minuteword != "" || ToPastWord != "" || HalfWord != "") {
    Minuteword = "";
    ToPastWord = "";
    HalfWord = "";
    TimeWordChanged = true;
    } 
 }
 
// Now we look for the Timeword. As the Minute changes with it we need not assign the TImeWordChanged Bool.
int Hour=0;
Hour = CharToInt(timeHourTwelve[0], timeHourTwelve[1]); 
// If we want a 24-hour CLock we replace timeHourTwelve with TimeHour
//Serial.print (Hour);
if (Hour == 1 && HourWord != "eins") HourWord = "eins";
if (Hour == 2 && HourWord != "zwei") HourWord = "zwei";
if (Hour == 3 && HourWord != "drei") HourWord = "drei";
if (Hour == 4 && HourWord != "vier") HourWord = "vier";
if (Hour == 5 && HourWord != "fünf") HourWord = "fünf";
if (Hour == 6 && HourWord != "sechs") HourWord = "sechs";
if (Hour == 7 && HourWord != "sieben") HourWord = "sieben";
if (Hour == 8 && HourWord != "acht") HourWord = "acht";
if (Hour == 9 && HourWord != "neun") HourWord = "neun";
if (Hour == 10 && HourWord != "zehn") HourWord = "zehn";
if (Hour == 11 && HourWord != "elf") HourWord = "elf";
if (Hour == 12 && HourWord != "zwölf") HourWord = "zwölf";
if (Hour == 13 && HourWord != "dreizehn") HourWord = "dreizehn";
if (Hour == 14 && HourWord != "vierzehn") HourWord = "vierzehn";
if (Hour == 15 && HourWord != "fünfzehn") HourWord = "fünfzehn";
if (Hour == 16 && HourWord != "sechzehn") HourWord = "sechzehn";
if (Hour == 17 && HourWord != "siebzehn") HourWord = "siebzehn";
if (Hour == 18 && HourWord != "achtzehn") HourWord = "achtzehn";
if (Hour == 19 && HourWord != "neunzehn") HourWord = "neunzehn";
if (Hour == 20 && HourWord != "zwanzig") HourWord = "zwanzig";
if (Hour == 21 && HourWord != "einundzwanzig") HourWord = "einundzwanzig";
if (Hour == 22 && HourWord != "zweiundzwanzig") HourWord = "zweiundzwanzig";
if (Hour == 23 && HourWord != "dreiundzwanzig") HourWord = "dreiundzwanzig";
if (Hour == 24 && HourWord != "vierundzwanzig") HourWord = "vierundzwanzig";
}

// MapWordstoPixel maps the current Hour-Words to the corresponding Pixels.
void MapWordstoPixel() {
// Here my Letter-Matrix
//   1234567890
// 0 ESISTxKURZ 
// 1 FÜNFxxZEHN
// 2 VIERTELVOR
// 3 NACHxHALBx
// 4 EINSxZWEIx
// 5 xDREIxVIER
// 6 FÜNFxSECHS
// 7 SIEBENxELF
// 8 NEUNxACHTx
// 9 ZEHNxZWÖLF
//10 UNDZWANZIG

uint8_t _EsIst[5] = {1,2,3,4,5};
uint8_t _Mempty[7] = {255,255,255,255,255,255,255};
uint8_t _Kurz[7] = {7,8,9,10,10,10,10};
uint8_t _MFuenf[7] = {11,12,13,14,14,14,14};
uint8_t _MZehn[7] = {17,18,19,20,20,20,20};
uint8_t _MViertel[7] = {21,22,23,24,25,26,27};
uint8_t _halbempty[4] = {255,255,255,255};
uint8_t _halb[4] = {36,37,38,39};
uint8_t _vor[4] = {28,29,30,30};
uint8_t _nach[4] = {31,32,33,34};
uint8_t _eins[6] = {41,42,43,44,44,44};
uint8_t _zwei[6] = {46,47,48,49,49,49};
uint8_t _drei[6] = {52,53,54,55,55,55};
uint8_t _vier[6] = {57,58,59,60,60,60};
uint8_t _fuenf[6] = {61,62,63,64,64,64};
uint8_t _sechs[6] = {66,67,68,69,70,70};
uint8_t _sieben[6] = {71,72,73,74,75,76};
uint8_t _acht[6] = {86,87,88,89,89,89};
uint8_t _neun[6] = {81,82,83,84,84,84};
uint8_t _zehn[6] = {91,92,93,94,94,94};
uint8_t _elf[6] = {78,79,80,80,80,80};
uint8_t _zwoelf[6] = {96,97,98,99,100};
if (ItIs == "Es ist") {memcpy(ItIsInt, _EsIst, sizeof(_EsIst));}
if (Minuteword == "") {memcpy(MinuteInt, _Mempty, sizeof(_Mempty));}
if (Minuteword == "kurz") {memcpy(MinuteInt, _Kurz, sizeof(_Kurz));}
if (Minuteword == "fünf") {memcpy(MinuteInt, _MFuenf, sizeof(_MFuenf));}
if (Minuteword == "zehn") {memcpy(MinuteInt, _MZehn, sizeof(_MZehn));}
if (Minuteword == "viertel") {memcpy(MinuteInt, _MViertel, sizeof(_MViertel));}
if (ToPastWord == "") {memcpy(ToPastInt, _halbempty, sizeof(_halbempty));}
if (ToPastWord == "vor") {memcpy(ToPastInt, _vor, sizeof(_vor));}
if (ToPastWord == "nach") {memcpy(ToPastInt, _nach, sizeof(_nach));}
if (HalfWord == "") {memcpy(HalfInt, _halbempty, sizeof(_halbempty));}
if (HalfWord == "halb") {memcpy(HalfInt, _halb, sizeof(_halb));}
if (HourWord == "eins") {memcpy(HourInt, _eins, sizeof(_eins));}
if (HourWord == "zwei") {memcpy(HourInt, _zwei, sizeof(_zwei));}
if (HourWord == "drei") {memcpy(HourInt, _drei, sizeof(_drei));}
if (HourWord == "vier") {memcpy(HourInt, _vier, sizeof(_vier));}
if (HourWord == "fünf") {memcpy(HourInt, _fuenf, sizeof(_fuenf));}
if (HourWord == "sechs") {memcpy(HourInt, _sechs, sizeof(_sechs));}
if (HourWord == "sieben") {memcpy(HourInt, _sieben, sizeof(_sieben));}
if (HourWord == "acht") {memcpy(HourInt, _acht, sizeof(_acht));}
if (HourWord == "neun") {memcpy(HourInt, _neun, sizeof(_neun));}
if (HourWord == "zehn") {memcpy(HourInt, _zehn, sizeof(_zehn));}
if (HourWord == "elf") {memcpy(HourInt, _elf, sizeof(_elf));}
if (HourWord == "zwölf") {memcpy(HourInt, _zwoelf, sizeof(_zwoelf));}
}

// CharToInt converts the Chars of e.g. the Hour or the Minuteinfo to an Integer Value
int CharToInt(char _InChar1, char _InChar2) {
int _Inti =0;
String  _Instring ="";
_Instring.concat(_InChar1);
_Instring.concat(_InChar2);
_Inti = _Instring.toInt();
//Serial.println(_Inti);
return _Inti;
}

// Colorval2RGB converts the ColorValue 209010 to 20 Red 90 Green 10 Blue 
void Colorval2RGB(String _ColorValue, uint8_t & LEDRed, uint8_t & LEDGreen, uint8_t & LEDBlue){
  LEDColorValue.toCharArray(str,12);
  char *token ;
  token = strtok(str, DELIMITER);
  LEDRed = String(token).toInt();
 // Serial.println(LEDRed);
  token = strtok(NULL, DELIMITER);
  LEDGreen = String(token).toInt();
 // Serial.println(LEDGreen);
  token = strtok(NULL, DELIMITER);
  LEDBlue = String(token).toInt();
 // Serial.println(LEDBlue);
}

// Coordinate2Number converts a Matrixposition to the LED Number in your Chain
// Mine consists of a 10x10 Matrix soldered in one direction Left to Right, Bottom to Top.
int16_t Coordinate2Number(uint16_t _XPos, uint16_t _YPos) {
// Starting Point, Bottom Left is 1,1, to 10,10 in Up Right
int16_t _result = 0;
while (_XPos > 0 && _XPos < 10 && _YPos > 0 && _YPos < 10) {
  // Otherwise we have an error.
  _result = _XPos + (_YPos -1) *10;
  return _result;
  } 
}


//FlipBoard Flips the Board Right to Left
int8_t FlipBoard(uint8_t _OldNr) {
uint8_t _result = 255;
if (_OldNr > -1 && _OldNr < 254){
// If you have to mirror the board like I have to: 9 becomes 2 and 19 becomes 12
uint8_t _Row = 0;
uint8_t _Col = 0;
_Col = _OldNr %10;
_Row = _OldNr - _Col;
//Serial.print("_Oldnr :");
//Serial.print(_OldNr); 
_Col = 10 - _Col;
_result = _Row  + _Col;
//Serial.print(" - > New Nr. :");
//Serial.println(_result);
//Serial.print("_Row :");
//Serial.print(_Row); 
//Serial.print(", _Col :");
//Serial.println(_Col); 
// And we have to down the Result by 1 to be in the Old Matrix Staring with 0;
_result--;
}
return _result;
}

// Input a value 0 to 255 to get a color value.
// The colours are a transition r - g - b - back to r.
uint32_t Wheel(byte WheelPos) {
 WheelPos = 255 - WheelPos;
 if(WheelPos < 85) {
    return strip.Color(255 - WheelPos * 3, 0, WheelPos * 3);
    }
 if(WheelPos < 170) {
    WheelPos -= 85;
    return strip.Color(0, WheelPos * 3, 255 - WheelPos * 3);
    }
    WheelPos -= 170;
    return strip.Color(WheelPos * 3, 255 - WheelPos * 3, 0);
}

////////////////// FIRST MODE ////////////////////////////CHOSEN COLOR UNI WORD CLOCK ///////////////////

void MonoColorClock(String _ColorValue) {
   uint8_t _i = 0; // the Pointer to the Wanted LED
   uint8_t _LED = 0; // the Number of the current LED
  // Serial.println("We draw the Wordclock in the Selected Color");
   Colorval2RGB(_ColorValue, LEDRed, LEDGreen, LEDBlue);
    // We are set.
     strip.clear();
   // We draw the ItIs
//   Serial.println("Values for It Is :");
   for(_i=0;_i < sizeof(ItIsInt) ;_i++) {
      _LED= ItIsInt[_i]-1;
      _LED = FlipBoard(_LED);
//      Serial.print(_LED);
//      Serial.print(",");
      strip.setPixelColor(_LED,LEDRed, LEDGreen, LEDBlue);
      }
 //  Serial.println();
   // We draw the Minute
//   Serial.println("Values for Minute :");
   for(_i=0;_i < sizeof(MinuteInt) ;_i++) {
      _LED= MinuteInt[_i]-1;
      _LED = FlipBoard(_LED);
//      Serial.print(_LED);
//      Serial.print(",");
      strip.setPixelColor(_LED,LEDRed, LEDGreen, LEDBlue);
      }
 //  Serial.println();
   // We draw the ToPast
//   Serial.println("Values for ToPast :");
   for(_i=0;_i < sizeof(ToPastInt) ;_i++) {
      _LED= ToPastInt[_i]-1;
      _LED = FlipBoard(_LED);
//      Serial.print(_LED);
 //     Serial.print(",");
      strip.setPixelColor(_LED,LEDRed, LEDGreen, LEDBlue);
      }
 //   Serial.println();
   // We draw the Half
//  Serial.println("Values for Half :");
   for(_i=0;_i < sizeof(HalfInt) ;_i++) {
      _LED= HalfInt[_i]-1;
 //     Serial.print(_LED);
 //     Serial.print(":");
      _LED = FlipBoard(_LED);
 //     Serial.print(_LED);
 //     Serial.print(",");
      strip.setPixelColor(_LED,LEDRed, LEDGreen, LEDBlue);
      }
  // Serial.println();
   // We draw the Hour
 //  Serial.println("Values for Hour :");
   for(_i=0;_i < sizeof(HourInt) ;_i++) {
      _LED= HourInt[_i]-1;
      _LED = FlipBoard(_LED);
 //     Serial.print(_LED);
 //     Serial.print(",");
      strip.setPixelColor(_LED,LEDRed, LEDGreen, LEDBlue);
      }
   strip.setBrightness(LEDBrightness);
   strip.show();
}
////////////////// SECOND MODE ////////////////////////////MULTICOLOR GLOBAL RAINBOW WORD CLOCK ///////////////////
void MultiColorClock(uint8_t _wait, uint8_t j) {
// REMINDER WHEEL -> 0..255 GIVES A COLORPOINT OF THE RAINBOW.
uint8_t _i = 0; // the Pointer to the Wanted LED
uint8_t _LED = 0; // the Number of the current LED
uint8_t _Min = 255;
uint8_t _Max = 0;
uint8_t _Length = 0;
strip.clear();
strip.setBrightness(LEDBrightness);
// It Is  
  for(_i=0;_i < sizeof(ItIsInt) ;_i++) {
      _LED= ItIsInt[_i]-1;
      _LED = FlipBoard(_LED);
      strip.setPixelColor(_LED, Wheel(((_LED * 256 / strip.numPixels()) + j) & 255));
      }
// Minute
 for(_i=0;_i < sizeof(MinuteInt) ;_i++) {
      _LED= MinuteInt[_i]-1;
      _LED = FlipBoard(_LED);
      strip.setPixelColor(_LED, Wheel(((_LED * 256 / strip.numPixels()) + j) & 255));
     }
// To Past
 for(_i=0;_i < sizeof(ToPastInt) ;_i++) {
      _LED= ToPastInt[_i]-1;
      _LED = FlipBoard(_LED);
      strip.setPixelColor(_LED, Wheel(((_LED * 256 / strip.numPixels()) + j) & 255));
      }
// Half
  for(_i=0;_i < sizeof(HalfInt) ;_i++) {
      _LED= HalfInt[_i]-1;
      _LED = FlipBoard(_LED);
      strip.setPixelColor(_LED, Wheel(((_LED * 256 / strip.numPixels()) + j) & 255));
     }
//Hour
 for(_i=0;_i < sizeof(HourInt) ;_i++) {
      _LED= HourInt[_i]-1;
      _LED = FlipBoard(_LED);
      strip.setPixelColor(_LED, Wheel(((_LED * 256 / strip.numPixels()) + j) & 255));
      }
   strip.show();
   delay(_wait*2);
} 


////////////////// THIRD MODE ////////////////////////////RAINBOWCYCLE WORD CLOCK ///////////////////
void RainbowColorClock(uint8_t _wait, uint8_t j, double _stretch) {
// REMINDER WHEEL -> 0..255 GIVES A COLORPOINT OF THE RAINBOW.
uint8_t _i = 0; // the Pointer to the Wanted LED
uint8_t _LED = 0; // the Number of the current LED
uint8_t _Min = 255;
uint8_t _Max = 0;
uint8_t _Length = 0;
if (_stretch < 1) {_stretch = 1;}
strip.clear();
strip.setBrightness(LEDBrightness);
// It Is  
  for(_i=0;_i < sizeof(ItIsInt) ;_i++) {
      if (_Min > ItIsInt[_i]-1){_Min =ItIsInt[_i]-1;}
      if (_Max < ItIsInt[_i]-1){_Max =ItIsInt[_i]-1;}
      }
  _Length = (1+_Max -_Min) * _stretch;
  for(_i=0;_i < sizeof(ItIsInt) ;_i++) {
      _LED= ItIsInt[_i]-1;
      _LED = FlipBoard(_LED);
      strip.setPixelColor(_LED, Wheel(((_LED * 256 / _Length) + j) & 255));
     // strip.setPixelColor(_LED, Wheel(((_LED * 256 / strip.numPixels()) + j) & 255));
      }
// Minute
  _Min=255;
  _Max=0;
  for(_i=0;_i < sizeof(MinuteInt) ;_i++) {
      if (_Min > MinuteInt[_i]-1){_Min =MinuteInt[_i]-1;}
      if (_Max < MinuteInt[_i]-1){_Max =MinuteInt[_i]-1;}
      }
      _Length = (1+_Max -_Min) * _stretch;
  for(_i=0;_i < sizeof(MinuteInt) ;_i++) {
      _LED= MinuteInt[_i]-1;
      _LED = FlipBoard(_LED);
       strip.setPixelColor(_LED, Wheel(((_LED * 256 / _Length) + j) & 255));
     }
// To Past
  _Min=255;
  _Max=0;
  for(_i=0;_i < sizeof(ToPastInt) ;_i++) {
    if (_Min > ToPastInt[_i]-1){_Min =ToPastInt[_i]-1;}
    if (_Max < ToPastInt[_i]-1){_Max =ToPastInt[_i]-1;}
    }
    _Length = (1+_Max -_Min) * _stretch;
  for(_i=0;_i < sizeof(ToPastInt) ;_i++) {
      _LED= ToPastInt[_i]-1;
      _LED = FlipBoard(_LED);
      strip.setPixelColor(_LED, Wheel(((_LED * 256 / _Length) + j) & 255));
     }
// Half
  _Min=255;
  _Max=0;
  for(_i=0;_i < sizeof(HalfInt) ;_i++) {
      if (_Min > HalfInt[_i]-1){_Min =HalfInt[_i]-1;}
      if (_Max < HalfInt[_i]-1){_Max =HalfInt[_i]-1;}
      }
      _Length = (1+_Max -_Min) * _stretch;
  for(_i=0;_i < sizeof(HalfInt) ;_i++) {
      _LED= HalfInt[_i]-1;
      _LED = FlipBoard(_LED);
      strip.setPixelColor(_LED, Wheel(((_LED * 256 / _Length) + j) & 255));
     }
//Hour
  _Min=255;
  _Max=0;
  for(_i=0;_i < sizeof(HourInt) ;_i++) {
      if (_Min > HourInt[_i]-1){_Min =HourInt[_i]-1;}
      if (_Max < HourInt[_i]-1){_Max =HourInt[_i]-1;}
      }
      _Length = (1+_Max -_Min) * _stretch;
  for(_i=0;_i < sizeof(HourInt) ;_i++) {
      _LED= HourInt[_i]-1;
      _LED = FlipBoard(_LED);
      strip.setPixelColor(_LED, Wheel(((_LED * 256 / _Length) + j) & 255));
      }
   strip.show();
   delay(_wait);
} 
//// FOURTH MODE INVERTED WORD CLOCK ////////////////////////////////////////////////////////////
void InvertedColorClock(uint8_t _wait, uint8_t j) {
uint8_t _i = 0; // the Pointer to the Wanted LED
uint8_t _LED = 0; // the Number of the current LED
strip.clear();
strip.setBrightness(LEDBrightness);
// First we color the whole Background in a Rainbow
  for(_i=0;_i < strip.numPixels()
  ;_i++) {
    strip.setPixelColor(_i, Wheel(((_i * 256 / strip.numPixels()) + j) & 255));
  }
// Now we color the Rest in the Selected Color.
// It Is  
  for(_i=0;_i < sizeof(ItIsInt) ;_i++) {
      _LED= ItIsInt[_i]-1;
      _LED = FlipBoard(_LED);
      strip.setPixelColor(_LED,LEDRed, LEDGreen, LEDBlue);
      }
// Minute
 for(_i=0;_i < sizeof(MinuteInt) ;_i++) {
      _LED= MinuteInt[_i]-1;
      _LED = FlipBoard(_LED);
      strip.setPixelColor(_LED,LEDRed, LEDGreen, LEDBlue);
     }
// To Past
 for(_i=0;_i < sizeof(ToPastInt) ;_i++) {
      _LED= ToPastInt[_i]-1;
      _LED = FlipBoard(_LED);
      strip.setPixelColor(_LED,LEDRed, LEDGreen, LEDBlue);
      }
// Half
  for(_i=0;_i < sizeof(HalfInt) ;_i++) {
      _LED= HalfInt[_i]-1;
      _LED = FlipBoard(_LED);
      strip.setPixelColor(_LED,LEDRed, LEDGreen, LEDBlue);
     }
//Hour
 for(_i=0;_i < sizeof(HourInt) ;_i++) {
      _LED= HourInt[_i]-1;
      _LED = FlipBoard(_LED);
      strip.setPixelColor(_LED,LEDRed, LEDGreen, LEDBlue);
      }
   strip.show();
   delay(_wait*2);
} 
////////////////// Fifth MODE Jans Special/////////////////MULTICOLOR GLOBAL RAINBOW WORD CLOCK WITH DROPS///////////////////
void JanColorClock(uint8_t _wait, uint8_t j) {
// REMINDER WHEEL -> 0..255 GIVES A COLORPOINT OF THE RAINBOW.
uint8_t _i = 0; // the Pointer to the Wanted LED
uint8_t _LED = 0; // the Number of the current LED
uint8_t _Min = 255;
uint8_t _Max = 0;
uint8_t _Length = 0;
strip.clear();
strip.setBrightness(LEDBrightness);
// It Is  
  for(_i=0;_i < sizeof(ItIsInt) ;_i++) {
      _LED= ItIsInt[_i]-1;
      _LED = FlipBoard(_LED);
      strip.setPixelColor(_LED, Wheel(((_LED * 256 / strip.numPixels()) + j) & 255));
      }
// Minute
 for(_i=0;_i < sizeof(MinuteInt) ;_i++) {
      _LED= MinuteInt[_i]-1;
      _LED = FlipBoard(_LED);
      strip.setPixelColor(_LED, Wheel(((_LED * 256 / strip.numPixels()) + j) & 255));
     }
// To Past
 for(_i=0;_i < sizeof(ToPastInt) ;_i++) {
      _LED= ToPastInt[_i]-1;
      _LED = FlipBoard(_LED);
      strip.setPixelColor(_LED, Wheel(((_LED * 256 / strip.numPixels()) + j) & 255));
      }
// Half
  for(_i=0;_i < sizeof(HalfInt) ;_i++) {
      _LED= HalfInt[_i]-1;
      _LED = FlipBoard(_LED);
      strip.setPixelColor(_LED, Wheel(((_LED * 256 / strip.numPixels()) + j) & 255));
     }
//Hour
 for(_i=0;_i < sizeof(HourInt) ;_i++) {
      _LED= HourInt[_i]-1;
      _LED = FlipBoard(_LED);
      strip.setPixelColor(_LED, Wheel(((_LED * 256 / strip.numPixels()) + j) & 255));
      }
   strip.show();
   delay(_wait*2);
// The Upper Part was similar to mode 2. Now we Add some Circles.

} 


// The unsused LED Effects //////////////////////////////////////////////////////////////////////////////////
void LEDTest(uint16_t _Min, uint16_t _Max, uint16_t _Brightness, String _ColorValue){
// Local Variables get the underline first.
  int _x = 0;
  int _i = 0;
  Colorval2RGB(_ColorValue, LEDRed, LEDGreen, LEDBlue);
   for(_i=_Min;_i < _Max ;_i++) {
   strip.clear();
   strip.show();
    for(_x=0;_x <= _Brightness ; _x=_x+20) {
      strip.setBrightness(_x);
      strip.setPixelColor(_i, LEDRed, LEDGreen, LEDBlue);
      strip.show();
      delay(10);
      }
   strip.clear(); // Setzt alle Pixel auf 0   
   strip.show();
   } 
}

//Theatre-style crawling lights with rainbow effect
void theaterChaseRainbow(uint8_t _wait, uint8_t _MinPixel,  uint8_t _MaxPixel) {
 for (int _j=0; _j < 256; _j++) { // cycle all 256 colors in the wheel
  for (int _q=0; _q < 3; _q++) {
    for (int _i=_MinPixel; _i < _MaxPixel; _i=_i+3) {
      strip.setPixelColor(_i+_q, Wheel( (_i+_j) % 255)); //turn every third pixel on
      }
    strip.show();
    delay(_wait);
    for (int _i=_MinPixel; _i < _MaxPixel; _i=_i+3) {
      strip.setPixelColor(_i+_q, 0); //turn every third pixel off
      }
    }
  }
}

// The Rainbow Cycle....
void rainbowCycle(uint8_t _wait, uint8_t _MinPixel, uint8_t _MaxPixel) {
uint16_t i, j;
for(j=0; j<256*2; j++) { // 5 cycles of all colors on wheel
  for(i=_MinPixel; i< _MaxPixel; i++) {
    strip.setPixelColor(i, Wheel(((i * 256 / strip.numPixels()) + j) & 255));
    }
  strip.show();
  delay(_wait);
  }
} 

// Lights up a single LED depending on the Bars and the colourpicker
void SingleLED(uint16_t _XPos, uint16_t _YPos, String _ColorValue) {
   Serial.println(" Set one LED was triggered");
   Colorval2RGB(_ColorValue, LEDRed, LEDGreen, LEDBlue);
   strip.clear();
   strip.setPixelColor(Coordinate2Number(_XPos, _YPos),LEDRed, LEDGreen, LEDBlue);
   strip.setBrightness(LEDBrightness);
   strip.show();
}
